library native_ads;

import 'dart:async';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

const _viewType = "native_admob";
enum NativeAdmobType { banner, full }

class NativeAdmob extends StatefulWidget {
  final String adUnitID;
  final NativeAdmobOptions options;
  final NativeAdmobType type;
  final int numberAds;

  final Widget loading;
  final Widget error;

  final NativeAdmobController controller;

  NativeAdmob({
     Key? key,
    required this.adUnitID,
    required this.options,
    required this.loading,
    required this.error,
    required this.controller,
    this.numberAds = 1,
    this.type = NativeAdmobType.full,
  })  : assert(adUnitID.isNotEmpty),
        super(key: key);

  @override
  _NativeAdmobState createState() => _NativeAdmobState();
}

class _NativeAdmobState extends State<NativeAdmob> {
  static final isAndroid = defaultTargetPlatform == TargetPlatform.android;
  static final isiOS = defaultTargetPlatform == TargetPlatform.iOS;

  NativeAdmobController? _nativeAdController;

  NativeAdmobOptions get _options => widget.options;

  NativeAdmobType get _type => widget.type;

  Widget get _loading =>
      widget.loading;

  Widget get _error => widget.error;

  var _loadState = AdLoadState.loading;
  StreamSubscription? _subscription;

  @override
  void initState() {
    _nativeAdController = widget.controller;
    _nativeAdController!
        .setAdUnitID(widget.adUnitID, numberAds: widget.numberAds);

    _subscription = _nativeAdController!.stateChanged.listen((state) {
      setState(() {
        _loadState = state;
      });
    });

    super.initState();
  }

  @override
  void dispose() {
    _subscription!.cancel();
    if (widget.controller == null) _nativeAdController!.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (isAndroid || isiOS) {
      return Container(
        child: _loadState == AdLoadState.loading
            ? _loading
            : _loadState == AdLoadState.loadError
                ? _error
                : _createPlatformView(),
      );
    }

    return Text('$defaultTargetPlatform is not supported PlatformView yet.');
  }

  Widget _createPlatformView() {
    final creationParams = {
      "options": _options.toJson(),
      "controllerID": _nativeAdController!.id,
      "type": _type.toString().replaceAll("NativeAdmobType.", ""),
    };

    return isAndroid
        ? AndroidView(
            viewType: _viewType,
            creationParamsCodec: const StandardMessageCodec(),
            creationParams: creationParams,
          )
        : UiKitView(
            viewType: _viewType,
            creationParamsCodec: const StandardMessageCodec(),
            creationParams: creationParams,
          );
  }
}
enum AdLoadState { loading, loadError, loadCompleted }

class NativeAdmobController {
  final _key = UniqueKey();

  String get id => _key.toString();

  final _stateChanged = StreamController<AdLoadState>.broadcast();

  Stream<AdLoadState> get stateChanged => _stateChanged.stream;

  /// Channel to communicate with plugin
  final _pluginChannel = const MethodChannel("flutter_native_admob");

  /// Channel to communicate with controller
  MethodChannel? _channel;
  String? _adUnitID;

  NativeAdmobController() {
    _channel = MethodChannel(id);
    _channel!.setMethodCallHandler(_handleMessages);

    // Let the plugin know there is a new controller
    _pluginChannel.invokeMethod("initController", {
      "controllerID": id,
    });
  }

  void dispose() {
    _pluginChannel.invokeMethod("disposeController", {
      "controllerID": id,
    });
  }

  Future<void> _handleMessages(MethodCall call) async {
    switch (call.method) {
      case "loading":
        _stateChanged.add(AdLoadState.loading);
        break;

      case "loadError":
        _stateChanged.add(AdLoadState.loadError);
        break;

      case "loadCompleted":
        _stateChanged.add(AdLoadState.loadCompleted);
        break;
    }
  }

  /// Change the ad unit ID
  void setAdUnitID(String adUnitID, {int numberAds = 1}) {
    _adUnitID = adUnitID;
    _channel!.invokeMethod("setAdUnitID", {
      "adUnitID": adUnitID,
      "numberAds": numberAds,
    });
  }

  /// Set the option to disable the personalized Ads
  void setNonPersonalizedAds(bool nonPersonalizedAds) {
    _channel!.invokeMethod("setNonPersonalizedAds", {
      "nonPersonalizedAds": nonPersonalizedAds,
    });
  }

  /// Reload new ad with specific native ad id
  ///
  ///  * [forceRefresh], force reload a new ad or using cache ad
  void reloadAd({bool forceRefresh = false, int numberAds = 1}) {
    if (_adUnitID == null) return;

    _channel!.invokeMethod("reloadAd", {
      "forceRefresh": forceRefresh,
      "numberAds": numberAds,
    });
  }

  void setTestDeviceIds(List<String> ids) {
    if (ids.isEmpty) return;

    _pluginChannel.invokeMethod("setTestDeviceIds", {
      "testDeviceIds": ids,
    });
  }
}
class NativeTextStyle {
  final double fontSize;
  final Color color;
  final Color backgroundColor;
  final bool isVisible;

  const NativeTextStyle({
    required this.fontSize,
    required this.color,
    required this.backgroundColor,
    this.isVisible = true,
  });

  Map<String, dynamic> toJson() => {
    "backgroundColor": backgroundColor != null
        ? "#${backgroundColor.value.toRadixString(16)}"
        : null,
    "fontSize": fontSize,
    "color": color != null ? "#${color.value.toRadixString(16)}" : null,
    "isVisible": isVisible,
  };
}

class NativeAdmobOptions {
  final bool showMediaContent;
  final Color ratingColor;
  final NativeTextStyle adLabelTextStyle;
  final NativeTextStyle headlineTextStyle;
  final NativeTextStyle advertiserTextStyle;
  final NativeTextStyle bodyTextStyle;
  final NativeTextStyle storeTextStyle;
  final NativeTextStyle priceTextStyle;
  final NativeTextStyle callToActionStyle;

  const NativeAdmobOptions({
    this.showMediaContent = true,
    this.ratingColor = Colors.yellow,
    this.adLabelTextStyle = const NativeTextStyle(
      fontSize: 12,
      color: Colors.white,
      backgroundColor: Color(0xFFFFCC66),
    ),
    this.headlineTextStyle = const NativeTextStyle(
      fontSize: 16,
      color: Colors.black,
      backgroundColor: Colors.white,
    ),
    this.advertiserTextStyle = const NativeTextStyle(
      fontSize: 14,
      backgroundColor: Colors.white,
      color: Colors.black,
    ),
    this.bodyTextStyle = const NativeTextStyle(
      fontSize: 12,
      backgroundColor: Colors.white,
      color: Colors.grey,
    ),
    this.storeTextStyle = const NativeTextStyle(
      fontSize: 12,
      backgroundColor: Colors.white,
      color: Colors.black,
    ),
    this.priceTextStyle = const NativeTextStyle(
      fontSize: 12,
      backgroundColor: Colors.white,
      color: Colors.black,
    ),
    this.callToActionStyle = const NativeTextStyle(
      fontSize: 15,
      color: Colors.white,
      backgroundColor: Color(0xFF4CBE99),
    ),
  });

  Map<String, dynamic> toJson() => {
    "showMediaContent": showMediaContent,
    "ratingColor": "#${ratingColor.value.toRadixString(16)}",
    "adLabelTextStyle": adLabelTextStyle.toJson(),
    "headlineTextStyle": headlineTextStyle.toJson(),
    "advertiserTextStyle": advertiserTextStyle.toJson(),
    "bodyTextStyle": bodyTextStyle.toJson(),
    "storeTextStyle": storeTextStyle.toJson(),
    "priceTextStyle": priceTextStyle.toJson(),
    "callToActionStyle": callToActionStyle.toJson(),
  };
}